@@ -6,7 +6,7 @@ no warnings "redefine";
 
 use Data::Dumper;
 
-our $VERSION = '2.01';
+our $VERSION = '2.02';
 
 sub new {
   my $class = shift;
@@ -26,11 +26,15 @@ sub objecthook {
   my $struct = shift;
   my $code   = shift;
 
+  # Set up Data::Dumper to do the right thing
+  local $Data::Dumper::Freezer = 'Freeze';
+  local $Data::Dumper::Toaster = 'Thaw';
+  local $Data::Dumper::Bless = 'Pixie::Hook::Bless';
   my $datastring = $self->data_as_string( $struct );
-  use subs qw ( bless );
+  use subs qw ( Pixie::Hook::Bless );
   {
     my $VAR1;
-    local *bless = sub {
+    local *Bless = sub {
       my $structure = $_[0];
       my $package   = $_[1];
       return $_[0] = $code->( $structure, $package );
@@ -53,4 +57,13 @@ sub data_as_string {
   return Dumper( $struct );
 }
 
+# Set some 'catchers' for our Freeze and Thaw methods.
+sub UNIVERSAL::Freeze {
+    return $_[0];
+}
+
+sub UNIVERSAL::Thaw {
+    return $_[0];
+}
+
 1;
